{{ Nordic_nRF24L01_SpiTest110719a

  This checks to see if the Propeller can
  successfully communication with a nRF24L01+
  module via SPI.

  This program does not communicate with
  other nRF24L01+ molules.  Please use the
  other demos to test RF communication.
  
  This code is base on C code posted
  by Leon Heller in the Propeller Forum.
 
}} 
CON
  _clkmode = xtal1 + pll16x                           
  _xinfreq = 5_000_000
    
CON

  {
  'Pins to SparkFun Nordic nRF24L01+
  'Mecanum controller board
  _SpiIrq = 16
  _SpiMiso = 17
  _SpiMosi = 18
  _SpiSck = 19 
  _SpiCsn = 20     
  _SpiCe = 21
  }
  'Pins to Hong Kong Nordic nRF24L01+ (QuickStart board)
  _SpiIrq = 0
  _SpiMiso = 1
  _SpiMosi = 2
  _SpiSck = 3 
  _SpiCsn = 4     
  _SpiCe = 5
  {
  'Pins to SparkFun Nordic nRF24L01+ (QuickStart board)
  _SpiIrq = 0
  _SpiMiso = 1
  _SpiMosi = 2
  _SpiSck = 3 
  _SpiCsn = 4     
  _SpiCe = 5
  }
  

  '_RxLed = 0   ' set high to turn LED on
  '_TxLed = 1   ' set high to turn LED on 

  _LargestPayloadSize = 32
  _DefaultPayloadSize = _LargestPayloadSize
 
VAR

  byte payloadSize
  byte globalPayload[_LargestPayloadSize + 1] ' Add 1 so a terminating zero may be added if desired. 
  
OBJ   

  Debug : "FullDuplexSerialPlus"   
  Nordic : "NordicBeta110718a"  'Nordic nRF24L01+
  
PUB Demo | localIndex 


  Debug.start(31, 30, 0, 57600)

  'waitcnt(clkfreq * 2 + cnt)
  repeat while debug.rxcheck == -1
    if result // 40 == 0
      Debug.str(String(13, "Press any key to start."))
    waitcnt(clkfreq / 10 + cnt)
    result++
  Debug.str(String(13, "Start of Nordic SPI Check", 13))
  Debug.str(String(13, "This program does not communicate with other"))
  Debug.str(String(13, "Nordic modules.  This just checks the SPI"))
  Debug.str(String(13, "link between the Propeller chip and the"))
  Debug.str(String(13, "attached Nordic module.", 13))
  
  'dira[_RxLed]~~
  'dira[_TxLed]~~

  'Initialize Nordic nRF24L01
  dira[_SpiIrq]~

  payloadSize := byte[@configNew + 7] '_DefaultPayloadSize
  
  Nordic.Start(_SpiSck, _SpiMiso, _SpiMosi, _SpiCsn, _SpiCe, @configNew)
                           
   
  Debug.str(String(13, "Nordic Started", 13))
  
  repeat  
    SpiCheck
    Debug.str(String(13, 13, 13, "Press any key to start retest.", 13, 13))
    repeat while debug.rxcheck == -1
      
    
PUB SpiCheck | localIndex

  Debug.str(String(13, "Attempting to read rx address from module."))
  Nordic.SpiReadPub(Nordic#RX_ADDR_P0, @globalPayload, 5)
  Debug.str(String(13, "rx address = $"))
  repeat localIndex from 0 to 4
    Debug.hex(globalPayload[localIndex], 2)
  if CompareArrays(@globalPayload, @testAddress, 5) == -1
    Debug.str(String(13, "The present rx address is the same as the test address."))
    Debug.str(String(13, "Changing the test address from $"))
    repeat localIndex from 0 to 4
      Debug.hex(testAddress[localIndex], 2)
    Debug.str(String(", to $"))
    repeat localIndex from 0 to 4
      testAddress[localIndex] += $11
      Debug.hex(testAddress[localIndex], 2)
  Debug.str(String(13, "Attempting to write following rx address to module: $"))
  repeat localIndex from 0 to 4
    Debug.hex(testAddress[localIndex], 2)
  Nordic.SpiWritePub(Nordic#RX_ADDR_P0, @testAddress, 5)
  Debug.str(String(13, "Reading rx address from module: $"))
  Nordic.SpiReadPub(Nordic#RX_ADDR_P0, @globalPayload, 5)
  repeat localIndex from 0 to 4
    Debug.hex(globalPayload[localIndex], 2)
  if CompareArrays(@globalPayload, @testAddress, 5) == -1
    Debug.str(String(13, 13, "!!!!Success!!!!")) 
    Debug.str(String(13, 13, "The rx address was successfully changed.")) 
    Debug.str(String(13, "The test rx address will not necessarily be the rx address used")) 
    Debug.str(String(13, "in communicating with other modules.", 13)) 
  else
    Debug.str(String(13, 13, "** Failure **")) 
    Debug.str(String(13, 13, "The rx address was NOT changed.")) 
    Debug.str(String(13, "The Propeller was not able to communicate with the")) 
    Debug.str(String(13, "Nordic module.", 13)) 
  
PRI CompareArrays(localPtr0, localPtr1, localSize)

  result := -1
  repeat localSize
    if byte[localPtr0++] <> byte[localPtr1++]
      result := 0

DAT

testAddress   byte $AA, $BB, $CC, $DD, $EE



''------------------ Cofig settings for new nRF24L01+ modules ----------------------------------------
'' ----------------- Keep below together and in order ------------------------------------------------
configNew     byte %00111111 '%00111011  Register $00
              ' use %00111111($3F)(2 bytes CRC) or %00111011($3B)(1 byte CRC)
              ' when communicating with older nRF2401A modules
              ' make sure this matches nRF2401A setting
              ' %00001111($0F) future with more irq options
              ' 7 only zero allowed, 6 Mask irq by RX_DR (1 = no irq), 5 Mask TX_DS, 4 Mask MAX_RT,
              ' 3 Enable CRC, 2 CRC size (1 = 2 bytes), 1 1 = power up, 0 1 = PRX 0 = PTX
              ' RX_DR Receive Data Ready
              ' TX_DS Transmit Data Sent
              ' MAX_RT Maximum Retries
              byte $00 '$03 ' Register $01 Auto Acknowledge
              ' use $00 when communicating with older nRF2401A modules
              ' %00111111 future
              ' 7:6 only %00, enable pipe 5 through 0
              byte %00000011 ' Register $02
              ' 7:6 only zero allowed, 5:0 enable rx addresses pipe 5:0
              byte $03 ' Register $03 SETUP_AW setup Address Widths 
              ' 7:2 only %000000, 1:0 %01 = 3 %11 = 5
              byte $00 ' Register $04 SETUP_RETR Setup Auto Retransmission
              ' use $00 when communicating with older nRF2401A modules     
              ' %00101111 = 750us delay to retx, tx 15 tries
              ' %01001111 = 1250us delay to retx, tx 15 tries
              ' 7:4 ARD delay 1111 = 4000us, 3:0 retry count 0 - 15 (1111)
              byte $02 ' Register $05 RF_CH RF Channel
              ' 7 leave 0, 6:0 frequency channel
              ' F = (2400 + RF_CH)MHz (both nRF24L01+ and nRF2401A modules)
              byte $06 ' Register $06 RF_SETUP RF Setup
              ' don't use 2Mbps when communicating with older nRF2401A modules
              ' $06 = 1Mbps and full power
              ' 7:6 %00 only, 5 RF_DR_LOW 1 = 250kbps use 0 for higher speeds, 4 leave %0
              ' 3 1 = 2Mbps 0 = 1Mbps, 2:1 RF_POWER %11 = 0dBm (highest power), 0 don't care
              ' Register $07 STATUS Status Register used below.
              ' Register $08 OBSERVE_TX 7:4 lost packet count (read only),
              ' 3:0 retransmitted packets count (read only).  I plan to use this
              ' register's infomation in the future.
              ' Register $09 RPD 7:1 reserved (read only), 0 Received Power Detector (read only)
              ' Register $0A RX_ADDR_P0 39:0 Receive address pipe 0, 5 bytes maximum LSByte written first
              ' used below rxAddress0 and rxAddress1
              ' Register $0B RX_ADDR_P1 39:0 Receive address pipe 1
              ' Register $0C RX_ADDR_P2 7:0 Receive address pipe 2
              ' Register $0D RX_ADDR_P3 7:0 Receive address pipe 3
              ' Register $0E RX_ADDR_P4 7:0 Receive address pipe 4
              ' Register $0F RX_ADDR_P5 7:0 Receive address pipe 5
              ' Register $10 TX_ADDR 39:0 Transmit address
'configNew[7]
nordicStatus  byte %01000000 ' $40   %01110000   '$70
              ' only zero allowed, Data Ready RX, TX ACK received, MAX_RT failed,
              ' 3:1 Data pipe with payload %111 if empty (read only), 0 1= TX FIFO full (read only)
rxPayload     byte 20 '25 '32   ' Register $11 RX_PW_P0
              ' This will need to be reduced to 25 (assuming 2 bytes CRC and 5 bytes address)
              ' to work with older nRF2401 modules.
              ' Newer nRF24L01+ modules can have payloads as large as 32 bytes.
              ' 7:6 only zero allowed, 5:0 Number of bytes in RX payload in data pipe 0
              ' (1 to 32)(0 = pipe not used)
              ' Register $12 RX_PW_P1 7:6 only zero allowed,
              ' 5:0 Number of bytes in RX payload in data pipe 1
              ' (1 to 32)(0 = pipe not used)
              ' Register $13 RX_PW_P2 7:6 only zero allowed,
              ' 5:0 Number of bytes in RX payload in data pipe 2
              ' Register $14 RX_PW_P3 7:6 only zero allowed,
              ' 5:0 Number of bytes in RX payload in data pipe 3
              ' Register $15 RX_PW_P4 7:6 only zero allowed,
              ' 5:0 Number of bytes in RX payload in data pipe 4
              ' Register $16 RX_PW_P5 7:6 only zero allowed,
              ' 5:0 Number of bytes in RX payload in data pipe 5
              ' Register $17 FIFO_STATUS 7 only zero allowed, 6 TX_REUSE (read only), 
              ' 5 TX_FULL %1 = full (read only), 4 TX_EMPTY %1 = empty (read only),
              ' 3:2 only zero allowed, 1 RX_FULL %1 = full (read only),
              ' 0 RX_EMPTY %1 = empty (read only)
              ' Register $1C DYNPD Enable dynamic payload length, 7:6 only zero allowed,
              ' 5 DPL_P5 Enable dynamic payload length data pipe 5 (Requires EN_DPL and ENAA_5)
              ' 4 DPL_P4 Enable dynamic payload length data pipe 4 (Requires EN_DPL and ENAA_4)
              ' 3 DPL_P3 Enable dynamic payload length data pipe 3 (Requires EN_DPL and ENAA_3)
              ' 2 DPL_P2 Enable dynamic payload length data pipe 2 (Requires EN_DPL and ENAA_2)
              ' 1 DPL_P1 Enable dynamic payload length data pipe 1 (Requires EN_DPL and ENAA_1)
              ' 0 DPL_P0 Enable dynamic payload length data pipe 0 (Requires EN_DPL and ENAA_0)
              ' Register $1D FEATURE Feature Register 7:3 only zero allowed,
              ' 2 EN_DPL Enables Dynamic Payload Length, 1 EN_ACK_PAY Enables Payload with ACK,
              ' 0 EN_DYN_ACK Enables the W_TX_PAYLOAD_NOACK command

rxAddress     byte $E7[5] ' Register $0A Receive address data pipe 0, LSByte first
txAddress     byte $E7[5] ' Register $10 Transmit address data pipe 0, LSByte first
'' ----------------- Keep above together and in order ------------------------------------------------

               